#include "tcp.h"
#include "ui_tcp.h"

tcp::tcp(QWidget *parent) :
    QMainWindow(parent),
    ui(new Ui::tcp)
{
    ui->setupUi(this);
    ui->port->setText("8899");
    setWindowTitle("tcp设置");
    connectNum=0;
    ui->disconnect->setDisabled(true);
    ui->sendMsg->setEnabled(true);
    // 创建监听的服务器对象
    ms = new QTcpServer(this);
    connect(ms, &QTcpServer::newConnection, this, [=]() {
        QTcpSocket* clientSocket = ms->nextPendingConnection();
        mstatus->setPixmap(QPixmap(":/accessibility.svg").scaled(20, 20));

        ui->disconnect->setEnabled(true);

        // 将客户端连接添加到连接管理中
        connectedClients[clientSocket] = "";


        // 增加处理连接的信号和槽
        connectClient(clientSocket);

    });

    // 状态栏处理动作
    mstatus = new QLabel;
    // mstatus->setPixmap(QPixmap(qrc:/accessibility.svg).scaled(20,20));
    ui->statusbar->addWidget(new QLabel("是否有用户连接："));
    ui->statusbar->addWidget(mstatus);
}

tcp::~tcp()
{
    delete ui;
}

void tcp::on_setListen_clicked()
{
    unsigned short port = ui->port->text().toUShort();  // 获取端口号
    ms->listen(QHostAddress::Any, port);  // 监听所有地址上的指定端口
    ui->setListen->setDisabled(true);  // 禁用监听按钮

}

void tcp::connectClient(QTcpSocket *clientSocket)
{
    connectNum++;
    connect(clientSocket, &QTcpSocket::readyRead, this, [=]() {

        ui->usermsg->append("matlab连接");
        QByteArray data = clientSocket->readAll();
        if (data.isEmpty())
            return;

        char dataType = data.at(0);  // 读取前缀
        data.remove(0, 1);  // 移除前缀

        if (dataType == 'T') {
            // 处理文本数据
            QString message = QString::fromUtf8(data);
            ui->record->append("收到文本: " + message);
        } else if (dataType == 'D')  {
            // 处理数字数据
            QVector<double> receivedData;
            int dataSize = data.size() / sizeof(double);

            for (int i = 0; i < dataSize; ++i) {
                double value;
                memcpy(&value, data.data() + i * sizeof(double), sizeof(double));
                receivedData.append(value);
            }
            QString numbers;
            for (double num : receivedData)
            {
                numbers += QString::number(num, 'f', 6) + " ";
            }
            ui->record->append("收到数据: " + numbers.trimmed());
            QString dataSizeStr = QString::number(dataSize);
            ui->msg->append("收到数据长度: " + dataSizeStr);
            data.clear();  // 清空 data
        }
    });

    connect(clientSocket, &QTcpSocket::disconnected, this, [=]() {
        clientSocket->close();
        connectedClients.remove(clientSocket);
        connectNum = 0;

        // 更新状态栏
        if (connectNum == 0) {
            mstatus->setPixmap(QPixmap(":/f.PNG").scaled(20, 20));
            ui->usermsg->append("matlab断开连接");
        }
    });
}

void tcp::on_sendMsg_clicked()
{
    QString msg = ui->msg->toPlainText();  // 获取要发送的消息
    QString serverMsg = "[Server] " + msg;  // 添加服务器标识
    for (auto clientSocket : connectedClients.keys()) {
        if (clientSocket->state() == QAbstractSocket::ConnectedState) {
            clientSocket->write(serverMsg.toUtf8());  // 发送消息
        }
    }
    ui->record->append("服务端广播消息: " + msg);  // 在记录中显示发送的消息
}

